@@ -1,8 +1,9 @@
 # et:ts=4
 # package.tcl
-# $Id: package.tcl 43959 2008-12-18 08:46:30Z afb@macports.org $
+# $Id: package.tcl 79597 2011-06-19 20:59:11Z jmr@macports.org $
 #
-# Copyright (c) 2002 Apple Computer, Inc.
+# Copyright (c) 2004, 2007-2008, 2010 The MacPorts Project
+# Copyright (c) 2002-2004 Apple Inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -13,7 +14,7 @@
 # 2. Redistributions in binary form must reproduce the above copyright
 #    notice, this list of conditions and the following disclaimer in the
 #    documentation and/or other materials provided with the distribution.
-# 3. Neither the name of Apple Computer, Inc. nor the names of its contributors
+# 3. Neither the name of Apple Inc. nor the names of its contributors
 #    may be used to endorse or promote products derived from this software
 #    without specific prior written permission.
 # 
@@ -40,5 +41,5 @@ package require portdmg 1.0
 package require portmdmg 1.0
 package require portdpkg 1.0
 package require portportpkg 1.0
+package require portarchivefetch 1.0
 package require portunarchive 1.0
-package require portarchive 1.0
