@@ -1,8 +1,8 @@
 # et:ts=4
 # port.tcl
-# $Id: port.tcl 35249 2008-03-22 05:22:09Z wsiegrist@apple.com $
+# $Id: port.tcl 79597 2011-06-19 20:59:11Z jmr@macports.org $
 #
-# Copyright (c) 2002 Apple Computer, Inc.
+# Copyright (c) 2002 Apple Inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -13,7 +13,7 @@
 # 2. Redistributions in binary form must reproduce the above copyright
 #    notice, this list of conditions and the following disclaimer in the
 #    documentation and/or other materials provided with the distribution.
-# 3. Neither the name of Apple Computer, Inc. nor the names of its contributors
+# 3. Neither the name of Apple Inc. nor the names of its contributors
 #    may be used to endorse or promote products derived from this software
 #    without specific prior written permission.
 # 
@@ -43,7 +43,9 @@ package require portconfigure 1.0
 package require portbuild 1.0
 package require portdestroot 1.0
 package require portinstall 1.0
+package require portuninstall 1.0
 package require portactivate 1.0
+package require portdeactivate 1.0
 package require portclean 1.0
 package require porttest 1.0
 package require portlint 1.0
