@@ -1,7 +1,8 @@
 # registry.tcl
 #
+# Copyright (c) 2004-2005, 2007-2010 The MacPorts Project
 # Copyright (c) 2004 Will Barton <wbb4@opendarwin.org>
-# Copyright (c) 2002 Apple Computer, Inc.
+# Copyright (c) 2002 Apple Inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -12,7 +13,7 @@
 # 2. Redistributions in binary form must reproduce the above copyright
 #    notice, this list of conditions and the following disclaimer in the
 #    documentation and/or other materials provided with the distribution.
-# 3. Neither the name of Apple Computer, Inc. nor the names of its contributors
+# 3. Neither the name of Apple Inc. nor the names of its contributors
 #    may be used to endorse or promote products derived from this software
 #    without specific prior written permission.
 # 
@@ -34,16 +35,18 @@ package provide registry 1.0
 package require macports 1.0
 package require receipt_flat 1.0
 package require receipt_sqlite 1.0
-package require portimage 1.0
-package require portuninstall 1.0
+package require portimage 2.0
+package require registry_uninstall 2.0
 package require msgcat
 
 namespace eval registry {
+    variable lockfd
+    variable nlocked 0
 
 # Begin creating a new registry entry for the port version_revision+variant
 # This process assembles the directory name and creates a receipt dlist
 proc new_entry {name version {revision 0} {variants ""} {epoch 0} } {
-	global macports::registry.path macports::registry.format macports::registry.installtype macports::prefix
+	global macports::registry.path macports::registry.format macports::prefix
 
 	
 	# Make sure we don't already have an entry in the Registry for this
@@ -57,17 +60,10 @@ proc new_entry {name version {revision 0} {variants ""} {epoch 0} } {
 		property_store $ref revision $revision
 		property_store $ref variants $variants
 		property_store $ref epoch $epoch
-		# Trick to have a portable GMT-POSIX epoch-based time.
-		# (because we'll compare this with a file mtime).
-		property_store $ref date [expr [clock scan now -gmt true] - [clock scan "1970-1-1 00:00:00" -gmt true]]
-		property_store $ref installtype ${macports::registry.installtype}
+		property_store $ref date [clock seconds]
+		property_store $ref installtype image
 		property_store $ref receipt_f ${macports::registry.format}
-		if { ${macports::registry.installtype} == "image" } {
-			set imagedir [file join ${macports::registry.path} software ${name} ${version}_${revision}${variants}]
-			property_store $ref imagedir $imagedir
-			property_store $ref active 0
-			property_store $ref compact 0
-		}
+        property_store $ref active 0
 
 		return $ref
 	} else {
@@ -82,6 +78,12 @@ proc entry_exists {name version {revision 0} {variants ""}} {
 	return [${macports::registry.format}::entry_exists $name $version $revision $variants] 
 }
 
+# Check to see if any entry exists in the registry for the given port name.
+proc entry_exists_for_name {name} {
+	global macports::registry.format
+	return [${macports::registry.format}::entry_exists_for_name $name]
+}
+
 # Close the registry... basically wrap the receipts systems's write process
 proc write_entry {ref} {
 	global macports::registry.format
@@ -111,10 +113,10 @@ proc delete_entry {ref} {
 }
 
 # Open a registry entry.
-proc open_entry {name {version ""} {revision 0} {variants ""}} {
+proc open_entry {name {version ""} {revision 0} {variants ""} {epoch ""}} {
 	global macports::registry.format
 
-	return [${macports::registry.format}::open_entry $name $version $revision $variants]
+	return [${macports::registry.format}::open_entry $name $version $revision $variants $epoch]
 
 }
 
@@ -135,40 +137,36 @@ proc property_retrieve {ref property} {
 proc installed {{name ""} {version ""}} {
 	global macports::registry.format
 
-	set ilist [${macports::registry.format}::installed $name $version]
-	set rlist [list]
-
-	if { [llength $ilist] > 1 } {
-		foreach installed $ilist {
-			set iname [lindex $installed 0]
-			set iversion [lindex $installed 1]
-			set irevision [lindex $installed 2]
-			set ivariants [lindex $installed 3]
-			set iref [open_entry $iname $iversion $irevision $ivariants]
-			set iactive	[property_retrieve $iref active]
-			set iepoch [property_retrieve $iref epoch]
-			lappend rlist [list $iname $iversion $irevision $ivariants $iactive $iepoch]
-		}
-	} elseif { [llength $ilist] < 1 } {
-		if { $name == "" } {
-			return -code error "Registry error: No ports registered as installed."
-		} else {
-			if { $version == "" } {
-				return -code error "Registry error: $name not registered as installed."
-			} else {
-				return -code error "Registry error: $name $version not registered as installed."
-			}
-		}
-	} else {
-		set iname [lindex [lindex $ilist 0] 0]
-		set iversion [lindex [lindex $ilist 0] 1]
-		set irevision [lindex [lindex $ilist 0] 2]
-		set ivariants [lindex [lindex $ilist 0] 3]
-		set iref [open_entry $iname $iversion $irevision $ivariants]
-		set iactive	[property_retrieve $iref active]
-		set iepoch [property_retrieve $iref epoch]
-		lappend rlist [list $iname $iversion $irevision $ivariants $iactive $iepoch]
-	}
+    if {${macports::registry.format} == "receipt_flat"} {
+        set ilist [${macports::registry.format}::installed $name $version]
+        set rlist [list]
+    
+        foreach installed $ilist {
+            set iname [lindex $installed 0]
+            set iversion [lindex $installed 1]
+            set irevision [lindex $installed 2]
+            set ivariants [lindex $installed 3]
+            set iref [open_entry $iname $iversion $irevision $ivariants]
+            set iactive	[property_retrieve $iref active]
+            set iepoch [property_retrieve $iref epoch]
+            lappend rlist [list $iname $iversion $irevision $ivariants $iactive $iepoch]
+        }
+    } else {
+        set rlist [${macports::registry.format}::installed $name $version]
+    }
+    
+    if { [llength $rlist] < 1 } {
+        if { $name == "" } {
+            return -code error "Registry error: No ports registered as installed."
+        } else {
+            if { $version == "" } {
+                return -code error "Registry error: $name not registered as installed."
+            } else {
+                return -code error "Registry error: $name $version not registered as installed."
+            }
+        }
+    }
+    
 	return $rlist
 }
 
@@ -176,24 +174,26 @@ proc installed {{name ""} {version ""}} {
 # all ports if name is "").
 proc active {{name ""}} {
 	global macports::registry.format
-
-	set ilist [${macports::registry.format}::installed $name]
-	set rlist [list]
-
-	if { [llength $ilist] > 0 } {
-		foreach installed $ilist {
-			set iname [lindex $installed 0]
-			set iversion [lindex $installed 1]
-			set irevision [lindex $installed 2]
-			set ivariants [lindex $installed 3]
-			set iref [open_entry $iname $iversion $irevision $ivariants]
-			set iactive	[property_retrieve $iref active]
-			set iepoch [property_retrieve $iref epoch]
-			if {$iactive} {
-				lappend rlist [list $iname $iversion $irevision $ivariants $iactive $iepoch]
-			}
-		}
-	}
+    
+    if {${macports::registry.format} == "receipt_flat"} {
+        set rlist [list]
+        set ilist [${macports::registry.format}::installed $name]
+    
+        foreach installed $ilist {
+            set iname [lindex $installed 0]
+            set iversion [lindex $installed 1]
+            set irevision [lindex $installed 2]
+            set ivariants [lindex $installed 3]
+            set iref [open_entry $iname $iversion $irevision $ivariants]
+            set iactive	[property_retrieve $iref active]
+            set iepoch [property_retrieve $iref epoch]
+            if {$iactive} {
+                lappend rlist [list $iname $iversion $irevision $ivariants $iactive $iepoch]
+            }
+        }
+    } else {
+        set rlist [${macports::registry.format}::active $name]
+    }
 	
 	if { [llength $rlist] < 1 } {
 		if { $name == "" } {
@@ -255,6 +255,7 @@ proc register_bulk_files {files port} {
 	open_file_map
         set r [${macports::registry.format}::register_bulk_files $files $port]
 	write_file_map
+	close_file_map
 	return $r
 }
 
@@ -268,6 +269,11 @@ proc write_file_map {args} {
 	return [${macports::registry.format}::write_file_map $args]
 }
 
+proc close_file_map {args} {
+	global macports::registry.format
+	return [${macports::registry.format}::close_file_map $args]
+}
+
 # Dependency Map Code
 proc register_dependencies {deps name} {
 
@@ -349,15 +355,15 @@ proc fileinfo_for_index {flist} {
 }
 
 # List all ports this one depends on
-proc list_depends {name} {
+proc list_depends {name {version ""} {revision ""} {variants 0}} {
 	global macports::registry.format
-	return [${macports::registry.format}::list_depends $name]
+	return [${macports::registry.format}::list_depends $name $version $revision $variants]
 }
 
 # List all the ports that depend on this port
-proc list_dependents {name} {
+proc list_dependents {name {version ""} {revision ""} {variants 0}} {
 	global macports::registry.format
-	return [${macports::registry.format}::list_dependents $name]
+	return [${macports::registry.format}::list_dependents $name $version $revision $variants]
 }
 
 proc register_dep {dep type port} {
@@ -380,7 +386,118 @@ proc write_dep_map {args} {
 	return [${macports::registry.format}::write_dep_map $args]
 }
 
+# acquire exclusive lock on registry, do this before modifying it or reading
+# any info which will affect a decision on what to modify
+proc exclusive_lock {} {
+    global macports::registry.path
+    variable lockfd
+    variable nlocked
+    incr nlocked
+    if {$nlocked > 1} {
+        return
+    }
+    set lockpath [file join ${registry.path} registry .registry.lock]
+    if {![info exists lockfd]} {
+        if {![file writable [file dirname $lockpath]]} {
+            # skip locking, registry can't be modified anyway
+            return
+        }
+        set lockfd [::open $lockpath w]
+    }
+    if {[catch {flock $lockfd -exclusive -noblock} result]} {
+        if {$result == "EAGAIN"} {
+            ui_msg "Waiting for lock on $lockpath"
+            flock $lockfd -exclusive
+        } elseif {$result == "EOPNOTSUPP"} {
+            # Locking not supported, just return
+            ui_debug "flock not supported, not locking registry"
+        } else {
+            return -code error "$result obtaining lock on $lockpath"
+        }
+    }
+}
+
+# release exclusive lock on registry, do this when done writing to it
+proc exclusive_unlock {} {
+    variable lockfd
+    variable nlocked
+    incr nlocked -1
+    if {$nlocked > 0} {
+        return
+    } elseif {$nlocked < 0} {
+        ui_warn "exclusive_unlock called more often than exclusive_lock!"
+    }
+    if {[info exists lockfd]} {
+        # not much point trying to handle errors
+        catch {flock $lockfd -unlock}
+    }
+}
+
+# upgrade flat receipts to registry2.0 sqlite db
+proc convert_to_sqlite {} {
+    set ilist [receipt_flat::installed "" ""]
+
+    foreach installed $ilist {
+        set iname [lindex $installed 0]
+        set iversion [lindex $installed 1]
+        set irevision [lindex $installed 2]
+        set ivariants [lindex $installed 3]
+        set proplist [list name $iname version $iversion revision $irevision variants $ivariants]
+
+        set iref [receipt_flat::open_entry $iname $iversion $irevision $ivariants]
+
+        lappend proplist date [receipt_flat::property_retrieve $iref date]
+        lappend proplist epoch [receipt_flat::property_retrieve $iref epoch]
+        lappend proplist negated_variants [receipt_flat::property_retrieve $iref negated_variants]
+        lappend proplist requested [receipt_flat::property_retrieve $iref requested]
+        lappend proplist os_platform [receipt_flat::property_retrieve $iref os_platform]
+        lappend proplist os_major [receipt_flat::property_retrieve $iref os_major]
+        lappend proplist archs [receipt_flat::property_retrieve $iref archs]
+
+        set installtype [receipt_flat::property_retrieve $iref installtype]
+        lappend proplist installtype $installtype
+        set location [receipt_flat::property_retrieve $iref location]
+        if {$location == "0"} {
+            set location [receipt_flat::property_retrieve $iref imagedir]
+        }
+        set contents [receipt_flat::property_retrieve $iref contents]
+        set imagefiles [list]
+        set idlen [string length $location]
+        foreach f $contents {
+            set fullpath [lindex $f 0]
+            # strip image dir from start
+            if {[string range $fullpath 0 [expr $idlen - 1]] == $location} {
+                set path [string range $fullpath $idlen [string length $fullpath]]
+            } else {
+                set path $fullpath
+            }
+            lappend imagefiles $path
+        }
+        lappend proplist imagefiles $imagefiles
+        set active [receipt_flat::property_retrieve $iref active]
+        if {$active} {
+            set state installed
+            lappend proplist files [receipt_flat::port_registered $iname]
+        } else {
+            set state imaged
+        }
+        lappend proplist location $location
+        lappend proplist state $state
+        
+        receipt_flat::open_dep_map
+        set deplist [receipt_flat::list_depends $iname]
+        set depnames [list]
+        foreach dep $deplist {
+            lappend depnames [lindex $dep 0]
+        }
+        lappend proplist depends $depnames
+        
+        lappend proplist portfile ""
+        
+        # add the entry to the db
+        receipt_sqlite::create_entry_l $proplist
+    }
+}
 
 # End of registry namespace
 }
-
